#!/bin/python3
import re
import os
import sys
from datetime import datetime
import requests
from bs4 import BeautifulSoup

def parse_downloads(downloads_str):
    downloads_multipliers = {'K': 1000, 'M': 1000000, 'B': 1000000000}

    # Remove specific characters
    for char in [' ', '+', ',', '-', '<', '=', '>']:
        downloads_str = downloads_str.replace(char, '')

    if downloads_str[-1] in downloads_multipliers:
        return int(float(downloads_str[:-1]) * downloads_multipliers[downloads_str[-1]])
    else:
        return int(downloads_str)

def parse_size(size_str):
    size_multipliers = {
        'KB': 2**10,
        'MB': 2**20,
        'GB': 2**30,
        'TB': 2**40,
        'PB': 2**50,
        'EB': 2**60,
        'ZB': 2**70,
        'YB': 2**80,
        'RB': 2**90,
        'QB': 2**100
    }

    # Split the string into value and unit
    size_value_str, size_unit_str = size_str[:-2], size_str[-2:]

    if size_unit_str in size_multipliers:
        return float(size_value_str) * size_multipliers[size_unit_str]
    else:
        return float(size_value_str)

def is_md5(s):
    return re.fullmatch(r'[A-Fa-f0-9]{32}', s) is not None

def scrape_android_file_host(url):
    # Send an HTTP GET request
    response = requests.get(url)

    # Parse the HTML content
    soup = BeautifulSoup(response.content, "html.parser")

    # Extract the required information
    file_id = soup.find("input", {"name": "fid"})["value"]
    file_info_div = soup.find("div", class_="col-xs-8 col-xs-offset-4 file-info")
    filename = file_info_div.h3.text.strip()
    file_name, file_extension = os.path.splitext(filename)
    category_link = file_info_div.find("a", href=lambda href: href and "/?w=developers&did=" in href)
    category_id = category_link["href"].split("=")[-1] if category_link else None
    category = category_link.text.strip() if category_link else None
    user_link = file_info_div.find("a", href=lambda href: href and "/?w=profile&uid=" in href)
    user_id = user_link["href"].split("=")[-1] if user_link else None
    user = user_link.text.strip() if user_link else None
    downloads = soup.find("img", src="/images/d-s-downloads.png").find_next("span").text.strip()[:-9]
    downloads_approx = parse_downloads(downloads)
    size = soup.find("img", src="/images/d-s-size.png").find_next("span").text.strip()[:-4]
    size_approx = parse_size(size)
    md5 = soup.find("img", src="/images/d-s-md5.png").find_next("span").text.strip()[:-3]
    md5_check = is_md5(md5)
    md5 = md5 if md5_check else 'Unknown'
    upload_date = soup.find("img", src="/images/d-s-uploadeddate.png").find_next("span").text.strip()[:-11]
    upload_date_object = datetime.strptime(upload_date, '%b %d, %Y | %I:%M%p')
    year = upload_date_object.year
    year = f"{year:04d}"
    month = upload_date_object.month
    month = f"{month:02d}"
    day = upload_date_object.day
    day = f"{day:02d}"
    hour = upload_date_object.hour
    hour = f"{hour:02d}"
    minute = upload_date_object.minute
    minute = f"{minute:02d}"
    upload_date_iso_8601 = f"{year}-{month}-{day}T{hour}:{minute}±??:??"
    weekdays = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
    weekday_name = weekdays[upload_date_object.weekday()]
    months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
    month_name = months[int(month) - 1]
    upload_date_rfc_5322 = f"{weekday_name}, {day} {month_name} {year} {hour}:{minute}:?? ±????"
    upload_date_rfc_3339 = f"{year}-{month}-{day} {hour}:{minute}:??±??:??"
    description = soup.find("h4", string="Description:").find_next("span").text.strip()
    additional_information = soup.find("h4", string="Additional Information:").find_next("span").text.strip()

    # Print the extracted information
    print(f"File ID: {file_id}")
    print(f"File name: {file_name}")
    print(f"File extension: {file_extension}")
    print(f"Category ID: {category_id}")
    print(f"Category: {category}")
    print(f"User ID: {user_id}")
    print(f"User: {user}")
    print(f"Downloads: {downloads}")
    print(f"Downloads (approximate): {downloads_approx}")
    print(f"Size: {size}")
    print(f"Size (approximate): {size_approx}")
    print(f"MD5: {md5}")
    print(f"Upload Date: {upload_date}")
    print(f"Upload Date (ISO 8601): {upload_date_iso_8601}")
    print(f"Upload Date (RFC 5322): {upload_date_rfc_5322}")
    print(f"Upload Date (RFC 3339): {upload_date_rfc_3339}")
    print(f"Description: {description}")
    print(f"Additional Information: {additional_information}")

if __name__ == "__main__":
    # Check if URL argument is provided
    if len(sys.argv) < 2:
        script_name = sys.argv[0]
        print(f"Usage: {script_name} <URL>")
        sys.exit(1)

    # Extract URL from command line arguments
    url = sys.argv[1]

    # Scrape Android File Host website
    scrape_android_file_host(url)
